Library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;

entity edge_det_fsm is
	port(
		 ClkSlow    : in    std_logic;
		 SignalIn   : in    std_logic;  -- input
		 Reset		: in	std_logic;
		 PosEdge    : out   std_logic;  -- positive edge detected
		 NegEdge    : out   std_logic  -- negative edge detected
	    );
end edge_det_fsm;




architecture implementation of edge_det_fsm is

	type fsm_state is (S_Init, S_Low, S_High);
	signal present_state, next_state : fsm_state;

begin



	--Combinatorial logic 
	combinatorial: process(present_state, SignalIn)
	begin

		--Default values for the next_state, PosEdge and NegEdge values 
		next_state <= present_state;
		PosEdge <= '0';
		NegEdge <= '0';

		case present_state is

			when S_Init => 
				if SignalIn = '1' then
					next_state <= S_High;
				elsif SignalIn = '0' then
					next_state <= S_Low;
				end if;

			when S_Low =>
				if SignalIn = '1' then
					next_state <= S_High;
				end if;

			when S_High =>
				if SignalIn = '0' then
					next_state <= S_Low;
				end if;

			--Default action if in an invalid state
			when others => 
					next_state <= S_Init;


		end case;
	end process;


	--Synchronous Logic
	synchronous: process(ClkSlow, Reset)
	begin
		if Reset = '1' then
			present_state <= S_Init;
		elsif rising_edge(ClkSlow) then
			present_state <= next_state;
		end if;

	end process;

end;

