LIBRARY IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;	--required for the arithmetic with std_logic
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity event_counter is
    port (
            EventCtrOut : out std_logic_vector(3 downto 0);
            ClkSlow     : in std_logic;
            Reset       : in std_logic;
            SignalA     : in std_logic;
            SignalB     : in std_logic
         );
    end entity event_counter;



architecture a of event_counter is

    
    component edge_det is
    port(
         ClkSlow    : in    std_logic;
         SignalIn   : in    std_logic;  -- input
         PosEdge    : out   std_logic;  -- positive edge detected
         NegEdge    : out   std_logic  -- negative edge detected
         
     ); 
    end component;


    signal CountPos, CountNeg : std_logic_vector(1 downto 0);
    signal PosA, PosB, NegA, NegB : std_logic;

begin

    EDA: edge_det
    port map(
                ClkSlow => ClkSlow,
                SignalIn => SignalA,
                PosEdge => PosA,
                NegEdge => NegA
            );

    EDB: edge_det
    port map(
                ClkSlow => ClkSlow,
                SignalIn => SignalB,
                PosEdge => PosB,
                NegEdge => NegB
            );


    count_events: process(ClkSlow, Reset)
    begin
        if Reset = '1' then
          CountPos <= (others => '0');  
          CountNeg <= (others => '0');  
        elsif rising_edge(ClkSlow) then
			CountPos <= not CountPos;
			CountNeg <= not CountNeg;
        end if;

    end process;

    EventCtrOut <= CountNeg & CountPos;

end architecture a; 
