LIBRARY ieee;
USE ieee.std_logic_1164.all;


entity fpga_top is
	port (
		    --4x LEDs
			LED 	: out std_logic_vector(3 downto 0);

       		--40 MHz external clock pulse
			USER_CLOCK : in std_logic;

            --external asynchronous reset (high-active)
			USER_RESET : in std_logic;

		    --4x DIP Switches
			DIP		   : in std_logic_vector(3 downto 0)
		 );
	end entity fpga_top;


architecture behavorial of fpga_top is

    signal LogicOut, CounterOut, ShiftRegOut, EventCtrOut : std_logic_vector(3 downto 0);
    signal ClkSlow : std_logic;
    signal SEL : std_logic_vector(1 downto 0);


    --Component Declarations

    component clk_div is
    generic (Nd : Integer := 10); --PARAMETER BY HOW MUCH TO DIVIDE THE CLOCK
    port(
	    ClkIn 		: in  std_logic;
		ClkSlow     : out std_logic
    );
    end component;

    component counter is
    port(
         ClkSlow	: in  std_logic;
         Reset		: in  std_logic; -- here asynchronous reset
         Enable		: in  std_logic; -- enable
         CountDown	: in  std_logic; -- down when 1, else up
         CounterOut	: out std_logic_vector(3 downto 0)
    );
    end component;
    
    component logic_unit is
    port(
         a      : in    std_logic;
         b      : in    std_logic;
         LogOp1 : out	std_logic;
         LogOp2 : out	std_logic;
         LogOp3 : out	std_logic;
         LogOp4 : out	std_logic
     );
    end component;


    component mux4to1 is
    port(
        -- select input
        SEL  : in  std_logic_vector(1 downto 0);
        -- data inputs
        LogicOut		: in  std_logic_vector(3 downto 0);
        CounterOut		: in  std_logic_vector(3 downto 0);
        ShiftRegOut		: in  std_logic_vector(3 downto 0);
        EventCtrOut    	: in  std_logic_vector(3 downto 0);
        -- output
        LED				: out std_logic_vector(3 downto 0)
    );
    end component;


    component shift_reg is
    port(
         ClkSlow		: in  std_logic;
         Reset  		: in  std_logic; -- asynchronous reset
         enable  		: in  std_logic; -- enable
         ShiftRight 	: in  std_logic; -- direction
         ShiftRegOut	: out std_logic_vector(3 downto 0)
     );
    end component;

    component event_counter is
        port (
                EventCtrOut : out std_logic_vector(3 downto 0);
                ClkSlow     : in std_logic;
                Reset       : in std_logic;
                SignalA     : in std_logic;
                SignalB     : in std_logic
             );
    end component event_counter;


begin


    SEL <= DIP(2) & DIP(3);


    cdiv: clk_div 
    generic map(Nd => 10) --PARAMETER BY HOW MUCH TO DIVIDE THE CLOCK
    port map(
	    ClkIn 	    => USER_CLOCK,	
		ClkSlow     => ClkSlow
    );


   LGU: logic_unit 
   port map(
        LogOp1 => LogicOut(0),
        LogOp2 => LogicOut(1),
        LogOp3 => LogicOut(2),
        LogOp4 => LogicOut(3),
        A => DIP(0),
        B => DIP(1)
      );


   CTR : counter
   port map(
        CounterOut => CounterOut,
        ClkSlow => ClkSlow,
        Reset =>  USER_RESET,
        Enable =>  DIP(0),
        CountDown => DIP(1)
       );


   SHR : shift_reg
   port map(
        ShiftRegOut => ShiftRegOut,
        ClkSlow => ClkSlow,
        Reset =>  USER_RESET,
        Enable => DIP(0),
        ShiftRight => DIP(1)
       );



   EVN : event_counter
   port map(
        EventCtrOut =>  EventCtrOut,
        ClkSlow => ClkSlow,
        Reset => USER_RESET,
        SignalA => DIP(0),
        SignalB => DIP(1)
        );


   MUX : mux4to1
   port map(
        LED =>  LED,
        SEL => SEL,
        LogicOut => LogicOut,
        CounterOut => CounterOut,
        ShiftRegOut => ShiftRegOut,
        EventCtrOut => EventCtrOut
    );





end architecture behavorial;
